classdef GigeInterface < handle
  % GigeInterface
  %   Connect to a camera directly using the gige interface
  %
  % Methods:
  %   discover - Detect all the cameras on the network
  %   discoverOne - Returns a message of the first camera that responded
  %   find - Detect a specific camera on the network, will throw an error if it
  %          can't find the camera
  %   connect - Connect to the camera and gain exclusive access
  %   disconnect - Disconnect from the camera
  %   isConnected - Check whether a connection is still maintained with the
  %                 camera
  %   getStreamPacketSize - Get stream packet size
  %   setStreamPacketSize - Set stream packet size
  %   testStreamPacketSize - Test whether using the set packet size works
  %   getDelay - Get delay between network packets
  %   setDelay - Set delay between network packets
  %   downloadConfiguration - Download camera configuration from the camera
  %   downloadCalibration - Download calibration from the camera
  %   uploadConfiguration - Upload camera configuration to the camera
  %   uploadCalibration - Upload calibration to the camera
  %   depthStreamCapable - Check whether this camera is capable of streaming the
  %                        specified type
  %   startDepthStream - Start the depth stream
  %   stopDepthStream - Stop the depth stream
  %   hasDepthData - Returns true if a depth frame is available
  %   getDepthData - Get a depth frame
  %   imageStreamCapable - Check whether camera supports an image stream
  %   startImageStream - Start image stream
  %   stopImageStream - Stop image stream
  %   hasImageData - Check whether image frame is available
  %   getImageData - Get image frame
  %   getImageDataIntoPointer - Get the image frame into the supplied pointer
  %   persistentIpCapable - Get whether the camera is persistent ip capable
  %   getPersistentIp - Get the current persistent ip settings
  %   setPersistentIp - Set the persistent ip settings. The persistent ip
  %                     settings will be active on next reboot.
  %   getUserPointerCapacity - Get the capacity for amount of user pointers that
  %                            can be registered
  %   setUserPointerCapacity - Set the capacity for amount of user pointers that
  %                            can be registered
  %   addDepthUserPointer - Add a pointer that will be filled with depth data.
  %                         This method allows you to fill data straight into
  %                         your own data structure without any additional
  %                         copying.
  %   version - Get the tof library version running on the camera
  %   softwareTriggerCapable - Check whether the camera is capable of software
  %                            trigger
  %   softwareTrigger - Software trigger the camera
  %

  properties(Access=protected)
    mPtr = uint64(0);
  end

  methods(Access=public)
    function obj = GigeInterface(varargin)
      % GigeInterface
      %   Construct new gige_interface class
      %
      % ----------
      %
      %   Create an instance of the gige interface
      %
      % Inputs:
      %   1 - numeric - The port to use for GVCP communication, if 0 is
      %                 specified it will use an OS assigned port
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        obj.mPtr = varargin{1};
      elseif nargin == 1 && isa(varargin{1}, 'numeric')
        obj.mPtr = tof.mex('tof_gige_interface_new', varargin{1});
      elseif nargin == 0
        obj.mPtr = tof.mex('tof_gige_interface_new', 0);
      else
        error("Can't construct empty object");
      end
    end

    function delete(obj)
      if obj.mPtr ~= 0
        tof.mex('tof_gige_interface_delete', obj.mPtr);
        obj.mPtr = 0;
      end
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function ret_val = discover(obj)
      % discover
      %   Detect all the cameras on the network
      %
      % Output:
      %   ret_val - vector[DiscoveryMessage] - A list of discovery messages
      %

      ret_val = tof.mex('tof_gige_interface_discover', obj.mPtr);
      ret_val = arrayfun(@(vec_obj)tof.DiscoveryMessage(vec_obj), ret_val);
    end

    function ret_val = discoverOne(obj)
      % discoverOne
      %   Returns a message of the first camera that responded
      %
      % Output:
      %   ret_val - DiscoveryMessage - A discovery message
      %

      ret_val = tof.mex('tof_gige_interface_discover_one', obj.mPtr);
      ret_val = tof.DiscoveryMessage(ret_val);
    end

    function ret_val = find(obj, serial)
      % find
      %   Detect a specific camera on the network, will throw an error if it
      %   can't find the camera
      %
      % Inputs:
      %   serial - string - Serial number of camera to find
      %
      % Output:
      %   ret_val - DiscoveryMessage - A discovery message
      %

      ret_val = tof.mex('tof_gige_interface_find', obj.mPtr, serial);
      ret_val = tof.DiscoveryMessage(ret_val);
    end

    function connect(obj, message)
      % connect
      %   Connect to the camera and gain exclusive access
      %
      % Inputs:
      %   message - DiscoveryMessage - Discovery message of the camera to
      %                                connect to
      %

      tof.mex('tof_gige_interface_connect', obj.mPtr, message.get_object_ptr());
    end

    function disconnect(obj)
      % disconnect
      %   Disconnect from the camera
      %

      tof.mex('tof_gige_interface_disconnect', obj.mPtr);
    end

    function ret_val = isConnected(obj)
      % isConnected
      %   Check whether a connection is still maintained with the camera
      %
      % Output:
      %   ret_val - logical - Is connected
      %

      ret_val = tof.mex('tof_gige_interface_is_connected', obj.mPtr);
    end

    function ret_val = getStreamPacketSize(obj)
      % getStreamPacketSize
      %   Get stream packet size
      %
      % Output:
      %   ret_val - numeric - The packet size
      %

      ret_val = tof.mex('tof_gige_interface_get_stream_packet_size', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setStreamPacketSize(obj, packet_size)
      % setStreamPacketSize
      %   Set stream packet size
      %
      % Inputs:
      %   packet_size - numeric - The packet size
      %

      tof.mex('tof_gige_interface_set_stream_packet_size', obj.mPtr, packet_size);
    end

    function ret_val = testStreamPacketSize(obj, varargin)
      % testStreamPacketSize
      %   Test whether using the set packet size works
      %
      % Inputs:
      %   port = 0 - numeric - Port to send test packets to
      %
      % Output:
      %   ret_val - logical - Packet size works
      %

      port = 0;
      if size(varargin, 2) == 1
        port = varargin{1};
      end

      ret_val = tof.mex('tof_gige_interface_test_stream_packet_size', obj.mPtr, port);
    end

    function ret_val = getDelay(obj)
      % getDelay
      %   Get delay between network packets
      %
      % Output:
      %   ret_val - numeric - The delay, depending on the camera software it is
      %                       in nanoseconds or in cpu instruction
      %

      ret_val = tof.mex('tof_gige_interface_get_delay', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setDelay(obj, delay)
      % setDelay
      %   Set delay between network packets
      %
      % Inputs:
      %   delay - numeric - The delay, depending on the camera software it is in
      %                     nanoseconds or in cpu instruction
      %

      tof.mex('tof_gige_interface_set_delay', obj.mPtr, delay);
    end

    function ret_val = downloadConfiguration(obj, varargin)
      % downloadConfiguration
      %   Download camera configuration from the camera
      %
      % Inputs:
      %   port = 0 - numeric - Port to download configuration on
      %
      % Output:
      %   ret_val - CameraConfig - Camera configuration
      %

      port = 0;
      if size(varargin, 2) == 1
        port = varargin{1};
      end

      ret_val = tof.mex('tof_gige_interface_download_configuration', obj.mPtr, port);
      ret_val = tof.CameraConfig(ret_val);
    end

    function ret_val = downloadCalibration(obj, varargin)
      % downloadCalibration
      %   Download calibration from the camera
      %
      % Inputs:
      %   port = 0 - numeric - Port to download configuration on
      %
      % Output:
      %   ret_val - Calibration - Calibration
      %

      port = 0;
      if size(varargin, 2) == 1
        port = varargin{1};
      end

      ret_val = tof.mex('tof_gige_interface_download_calibration', obj.mPtr, port);
      ret_val = tof.Calibration(ret_val);
    end

    function uploadConfiguration(obj, config, varargin)
      % uploadConfiguration
      %   Upload camera configuration to the camera
      %
      % Inputs:
      %   config - CameraConfig - Camera configuration
      %   port = 0 - numeric - Port to download configuration on
      %

      port = 0;
      if size(varargin, 2) == 1
        port = varargin{1};
      end

      tof.mex('tof_gige_interface_upload_configuration', obj.mPtr, config.get_object_ptr(), port);
    end

    function ret_val = depthStreamCapable(obj, stream)
      % depthStreamCapable
      %   Check whether this camera is capable of streaming the specified type
      %
      % Inputs:
      %   stream - DepthStreamType - Depth stream type
      %
      % Output:
      %   ret_val - logical - Is capable
      %

      ret_val = tof.mex('tof_gige_interface_depth_stream_capable', obj.mPtr, uint32(stream));
    end

    function startDepthStream(obj, stream, varargin)
      % startDepthStream
      %   Start the depth stream
      %
      % Inputs:
      %   stream - DepthStreamType - Depth stream type
      %   port = 0 - numeric - Port to receive depth data on
      %

      port = 0;
      if size(varargin, 2) == 1
        port = varargin{1};
      end

      tof.mex('tof_gige_interface_start_depth_stream', obj.mPtr, uint32(stream), port);
    end

    function stopDepthStream(obj)
      % stopDepthStream
      %   Stop the depth stream
      %

      tof.mex('tof_gige_interface_stop_depth_stream', obj.mPtr);
    end

    function ret_val = hasDepthData(obj)
      % hasDepthData
      %   Returns true if a depth frame is available
      %
      % Output:
      %   ret_val - logical - Depth frame available
      %

      ret_val = tof.mex('tof_gige_interface_has_depth_data', obj.mPtr);
    end

    function ret_val = getDepthData(obj)
      % getDepthData
      %   Get a depth frame
      %
      % Output:
      %   ret_val - Data - Depth frame
      %

      ret_val = tof.mex('tof_gige_interface_get_depth_data', obj.mPtr);
      ret_val = tof.Data(ret_val);
    end

    function ret_val = imageStreamCapable(obj)
      % imageStreamCapable
      %   Check whether camera supports an image stream
      %
      % Output:
      %   ret_val - logical - Camera supports an image stream
      %

      ret_val = tof.mex('tof_gige_interface_image_stream_capable', obj.mPtr);
    end

    function startImageStream(obj, varargin)
      % startImageStream
      %   Start image stream
      %
      % Inputs:
      %   port = 0 - numeric - Port to receive image data on
      %

      port = 0;
      if size(varargin, 2) == 1
        port = varargin{1};
      end

      tof.mex('tof_gige_interface_start_image_stream', obj.mPtr, port);
    end

    function stopImageStream(obj)
      % stopImageStream
      %   Stop image stream
      %

      tof.mex('tof_gige_interface_stop_image_stream', obj.mPtr);
    end

    function ret_val = hasImageData(obj)
      % hasImageData
      %   Check whether image frame is available
      %
      % Output:
      %   ret_val - logical - Image frame available
      %

      ret_val = tof.mex('tof_gige_interface_has_image_data', obj.mPtr);
    end

    function ret_val = getImageData(obj)
      % getImageData
      %   Get image frame
      %
      % Output:
      %   ret_val - Data - Image data
      %

      ret_val = tof.mex('tof_gige_interface_get_image_data', obj.mPtr);
      ret_val = tof.Data(ret_val);
    end

    function ret_val = persistentIpCapable(obj)
      % persistentIpCapable
      %   Get whether the camera is persistent ip capable
      %
      % Output:
      %   ret_val - logical - Persistent IP capable
      %

      ret_val = tof.mex('tof_gige_interface_persistent_ip_capable', obj.mPtr);
    end

    function ret_val = getPersistentIp(obj)
      % getPersistentIp
      %   Get the current persistent ip settings
      %
      % Output:
      %   ret_val - PersistentIp - Persistent IP
      %

      ret_val = tof.mex('tof_gige_interface_get_persistent_ip', obj.mPtr);
      ret_val = tof.PersistentIp(ret_val);
    end

    function setPersistentIp(obj, persistent_ip)
      % setPersistentIp
      %   Set the persistent ip settings. The persistent ip settings will be
      %   active on next reboot.
      %
      % Inputs:
      %   persistent_ip - PersistentIp - The persistent ip
      %

      tof.mex('tof_gige_interface_set_persistent_ip', obj.mPtr, persistent_ip.get_object_ptr());
    end

    function ret_val = getUserPointerCapacity(obj)
      % getUserPointerCapacity
      %   Get the capacity for amount of user pointers that can be registered
      %
      % Output:
      %   ret_val - numeric - User pointer capacity
      %

      ret_val = tof.mex('tof_gige_interface_get_user_pointer_capacity', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setUserPointerCapacity(obj, capacity)
      % setUserPointerCapacity
      %   Set the capacity for amount of user pointers that can be registered
      %
      % Inputs:
      %   capacity - numeric - User pointer capacity
      %

      tof.mex('tof_gige_interface_set_user_pointer_capacity', obj.mPtr, capacity);
    end

    function ret_val = version(obj)
      % version
      %   Get the tof library version running on the camera
      %
      % Output:
      %   ret_val - string - Version
      %

      ret_val = tof.mex('tof_gige_interface_version', obj.mPtr);
    end

    function ret_val = softwareTriggerCapable(obj)
      % softwareTriggerCapable
      %   Check whether the camera is capable of software trigger
      %
      % Output:
      %   ret_val - logical - Software trigger capable
      %

      ret_val = tof.mex('tof_gige_interface_software_trigger_capable', obj.mPtr);
    end

    function softwareTrigger(obj)
      % softwareTrigger
      %   Software trigger the camera
      %

      tof.mex('tof_gige_interface_software_trigger', obj.mPtr);
    end

  end
end
