classdef KeaCamera < tof.Camera
  % KeaCamera
  %   The main interface to the kea camera
  %
  % Methods:
  %   onCameraProcessingCapable - Check whether this kea camera can do on camera
  %                               processing
  %   getOnCameraProcessing - Get on camera processing
  %   setOnCameraProcessing - Set on camera processing
  %   getDelay - Get delay between network packets
  %   setDelay - Set delay between network packets
  %   getPacketSize - Get maximum size of each network packet transmitted
  %   setPacketSize - Set maximum size of each network packet transmitted
  %   setCameraConfig - Set the camera config
  %   setConfigurations - Set both the camera and processing config
  %   version - Get the tof library version running on the camera
  %   softwareTriggerCapable - Check whether the camera is capable of software
  %                            trigger
  %   softwareTrigger - Software trigger the camera
  %

  methods(Access=public)
    function obj = KeaCamera(varargin)
      % KeaCamera
      %   Construct new kea_camera class
      %
      % ----------
      %
      %   Construct the kea camera
      %
      % Inputs:
      %   1 - ProcessingConfig - The processing config
      %   2 - string - The serial number of the camera to connect to, if the
      %                camera is available over both usb and gige it will be
      %                connected over usb. If an empty string is supplied the
      %                camera will connect to the first usb camera, if no usb
      %                camera is connected it will connect to the first gige
      %                camera that replies (this reply time is random, so you
      %                might connect to a different camera each time if you've
      %                got multiple cameras connected to your network). If no
      %                camera is found with the given serial number a nullptr
      %                and error is returned.
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        ptr = varargin{1};
      elseif nargin == 2 && isa(varargin{1}, 'tof.ProcessingConfig') && (isStringScalar(varargin{2}) || ischar(varargin{2}))
        ptr = tof.mex('tof_kea_camera_new', varargin{1}.get_object_ptr(), varargin{2});
      else
        error("Can't construct empty object");
      end

      obj@tof.Camera(ptr);

    end

    function delete(~)
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function ret_val = onCameraProcessingCapable(obj)
      % onCameraProcessingCapable
      %   Check whether this kea camera can do on camera processing
      %
      % Output:
      %   ret_val - logical - On camera processing capable
      %

      ret_val = tof.mex('tof_kea_camera_on_camera_processing_capable', obj.mPtr);
    end

    function ret_val = getOnCameraProcessing(obj)
      % getOnCameraProcessing
      %   Get on camera processing
      %
      % Output:
      %   ret_val - logical - True on camera processing, false on host
      %                       processing
      %

      ret_val = tof.mex('tof_kea_camera_get_on_camera_processing', obj.mPtr);
    end

    function setOnCameraProcessing(obj, on_camera_processing)
      % setOnCameraProcessing
      %   Set on camera processing
      %
      % Inputs:
      %   on_camera_processing - logical - True on camera processing, false on
      %                                    host processing
      %

      tof.mex('tof_kea_camera_set_on_camera_processing', obj.mPtr, on_camera_processing);
    end

    function ret_val = getDelay(obj)
      % getDelay
      %   Get delay between network packets
      %
      % Output:
      %   ret_val - numeric - The delay, depending on the camera software it is
      %                       in nanoseconds or in cpu instruction
      %

      ret_val = tof.mex('tof_kea_camera_get_delay', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setDelay(obj, delay)
      % setDelay
      %   Set delay between network packets
      %
      % Inputs:
      %   delay - numeric - The delay, depending on the camera software it is in
      %                     nanoseconds or in cpu instruction
      %

      tof.mex('tof_kea_camera_set_delay', obj.mPtr, delay);
    end

    function ret_val = getPacketSize(obj)
      % getPacketSize
      %   Get maximum size of each network packet transmitted
      %
      % Output:
      %   ret_val - numeric - Packet size in bytes
      %

      ret_val = tof.mex('tof_kea_camera_get_packet_size', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setPacketSize(obj, size)
      % setPacketSize
      %   Set maximum size of each network packet transmitted
      %
      % Inputs:
      %   size - numeric - Packet size in bytes
      %

      tof.mex('tof_kea_camera_set_packet_size', obj.mPtr, size);
    end

    function setCameraConfig(obj, camera_config)
      % setCameraConfig
      %   Set the camera config
      %
      % Inputs:
      %   camera_config - CameraConfig - Camera config
      %

      tof.mex('tof_kea_camera_set_camera_config', obj.mPtr, camera_config.get_object_ptr());
    end

    function setConfigurations(obj, camera_config, processing_config)
      % setConfigurations
      %   Set both the camera and processing config
      %
      % Inputs:
      %   camera_config - CameraConfig - Camera config
      %   processing_config - ProcessingConfig - Processing config
      %

      tof.mex('tof_kea_camera_set_configurations', obj.mPtr, camera_config.get_object_ptr(), processing_config.get_object_ptr());
    end

    function ret_val = version(obj)
      % version
      %   Get the tof library version running on the camera
      %
      % Output:
      %   ret_val - string - Version
      %

      ret_val = tof.mex('tof_kea_camera_version', obj.mPtr);
    end

    function ret_val = softwareTriggerCapable(obj)
      % softwareTriggerCapable
      %   Check whether the camera is capable of software trigger
      %
      % Output:
      %   ret_val - logical - Software trigger capable
      %

      ret_val = tof.mex('tof_kea_camera_software_trigger_capable', obj.mPtr);
    end

    function softwareTrigger(obj)
      % softwareTrigger
      %   Software trigger the camera
      %

      tof.mex('tof_kea_camera_software_trigger', obj.mPtr);
    end

  end
end
