import ctypes
from .__library__ import lib, get_error
from .data import *
from .stream import *
from .calibration import *
from .camera_config import *


class CsfReader():
    """
    This class reads csf files
    """
    def __del__(self):
        if self._ptr:
            lib.tof_csf_reader_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, file_location = None, ptr = None):
        """
        Construct new csf_reader class
        ----------

        Open a csf file for reading

        Parameters
        ----------
        file_location : str
                File location of the csf file
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif file_location != None:
            self._ptr = lib.tof_csf_reader_new(file_location.encode("utf-8"), get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def readFrame(self):
        """
        Read the next data frame from disk

        Returns
        ----------
        Data
            A data frame
        """
        return Data(ptr = lib.tof_csf_reader_read_frame(self._ptr, get_error()))


    def numberOfFrames(self):
        """
        Get the total number of frames in the csf file

        Returns
        ----------
        int
            Total number of frames in the csf file
        """
        return lib.tof_csf_reader_number_of_frames(self._ptr, get_error())


    def getHeader(self, index):
        """
        Returns an empty Data class containing the header information for the
        specified frame

        Parameters
        ----------
        index : int
            Which frame to get the header from

        Returns
        ----------
        Data
            Empty Data instance containing header information
        """
        return Data(ptr = lib.tof_csf_reader_get_header(self._ptr, index, get_error()))


    def nextStream(self):
        """
        Get the stream information of the next frame

        Returns
        ----------
        Stream
            Information on the next frame
        """
        return Stream(ptr = lib.tof_csf_reader_next_stream(self._ptr, get_error()))


    def nextFrameSize(self):
        """
        Get the size of the next frame

        Returns
        ----------
        int
            Size of the next frame
        """
        return lib.tof_csf_reader_next_frame_size(self._ptr, get_error())


    def frameIndex(self):
        """
        Get the current frame position

        Returns
        ----------
        int
            Frame index
        """
        return lib.tof_csf_reader_frame_index(self._ptr, get_error())


    def goToFrame(self, index):
        """
        Go to specified frame

        Parameters
        ----------
        index : int
            The index of the frame to jump to
        """
        return lib.tof_csf_reader_go_to_frame(self._ptr, index, get_error())


    def streams(self):
        """
        Get the streams that are in the csf file

        Returns
        ----------
        List[Stream]
            Streams in the csf file
        """
        size = lib.tof_csf_reader_streams(self._ptr, ctypes.c_void_p(), 0, get_error())
        array = (ctypes.c_void_p * size)()
        lib.tof_csf_reader_streams(self._ptr, array, size, get_error())
        return [Stream(ptr = item) for item in array]

    def streamCount(self, stream):
        """
        Get the number of frames contained in the file of the given stream

        Parameters
        ----------
        stream : Stream
            The stream

        Returns
        ----------
        int
            Number of frames for given stream
        """
        return lib.tof_csf_reader_stream_count(self._ptr, stream._ptr, get_error())


    def goToStream(self, stream, index):
        """
        Go to the specified stream, the next read frame call will start from here

        Parameters
        ----------
        stream : Stream
            The stream
        index : int
            The frame index
        """
        return lib.tof_csf_reader_go_to_stream(self._ptr, stream._ptr, index, get_error())


    def getCalibration(self):
        """
        Get calibration from the csf file

        Returns
        ----------
        Calibration
            Calibration
        """
        return Calibration(ptr = lib.tof_csf_reader_get_calibration(self._ptr, get_error()))


    def getCameraConfig(self):
        """
        Get camera config from the csf file

        Returns
        ----------
        CameraConfig
            Camera config
        """
        return CameraConfig(ptr = lib.tof_csf_reader_get_camera_config(self._ptr, get_error()))


